import re
from .base import BaseSearch


class Katcr(BaseSearch):
    """KickAssTorrents search engine.

    Search on any of the pirate bay proxies or thepiratebay itself if
    available. Extract torrents, description and return it tabulated and ready
    to be printed
    """

    proxy_name = 'kickass-torrents'
    url_format = '{}{}/search.php?q={}&p={}'
    url = 'https://kat.cd'

    @staticmethod
    def tabulate(link):
        """Make a given a href link parsed on bs4 printable."""
        magnet = link.find('a', title='Torrent magnet link')
        return (link.find(class_='cellMainLink').text,
                link.find_all('td')[1].text, magnet['href'])

    def get_torrents(self):
        """Return a list of torrents, printable."""
        # pylint: disable=not-callable
        torrents = self.browser.find_all(
            id=re.compile('torrent_latest_torrents(.*)'))
        return [self.tabulate(torrent) for torrent in torrents]



#!/usr/bin/env python3
"""Python library and CLI to search in multiple torrent sources.

Uses torrentmirror to get the first open mirror for any of the
supported sites.
"""

from contextlib import suppress
from pathlib import Path
import functools
import os
import shutil
import subprocess

from cleo import Command
from cleo import Application
from pygogo import Gogo
import cutie
import requests

from . import engines

DEFAULT_ENGINES = ['Katcr', 'ThePirateBay']
MAX_SIZE = shutil.get_terminal_size().columns - 20


@functools.lru_cache()
def get_engine(name, session, logger):
    """Cached engine extraction by name.

    Passing the logger down.
    """
    if not hasattr(engines, name):
        raise RuntimeError('Wrong engine provided')
    return getattr(engines, name)(session, logger)


class Result:
    def __init__(self, session, result, shortener, token, interactive):
        self.result = result
        self.session = session
        if shortener:
            shortener = shortener.format(os.getenv('KATCR_TOKEN', token))
        self.shortener = shortener
        self.is_interactive = interactive

    def __repr__(self):
        """Get new search res from shortened urls and adjust term output."""
        if not self.shortener:
            if self.is_interactive:
                return ' | '.join((self.result[0][:MAX_SIZE], self.result[1]))
            return ' | '.join(self.result)
        clean = self.result[:-1]
        result = self.session.post(self.shortener, data={'magnet': clean})
        return ''.join(clean) + f'{self.shortener}/{result.text}'

    def open(self):
        return subprocess.check_call(['xdg-open', self.result[-1]])


class CLICommand(Command):
    """Search in multiple torrent sites.

    search

        {search : Search term}

        {--pages=1 : Pages to search on search engines}
        {--token=? : Token to use on URL shortener as AUTH}
        {--shortener=? : URL Shortener}
        {--engines=Katcr,ThePirateBay,NyaaSi,Skytorrents : Engines}

        {--interactive=? : Allow the user to choose a specific magnet}
        {--open=? : Open selected magnet with xdg-open}

    """

    def handle(self):
        """Handler."""
        logger = Gogo(__name__, verbose=self.io.verbosity).logger
        session = requests.Session()
        session.verify = None

        engine_names = self.option('engines').split(',') or DEFAULT_ENGINES
        engines = (get_engine(a, session, logger) for a in engine_names)

        shortener = self.option('shortener')
        token = self.option('token')
        is_interactive = self.option('interactive')
        search_term = self.argument('search')
        pages = self.option('pages')

        search_res = []

        self.line('<info>Starting search on {}</info>'.format(
            ', '.join(engine_names)))

        # progress = self.progress_bar(len(engine_names))

        for engine in engines:
            # progress.advance(0.9)
            engine_result = engine.search(search_term, int(pages))
            search_res.extend((Result(session, a, shortener, token,
                                      is_interactive) for a in engine_result))

        # progress.finish()

        if not search_res:
            return

        if not is_interactive:
            return self.render_table(['Description', 'Size', 'Link'],
                                     [a.result for a in search_res])

        result = search_res[cutie.select(search_res, selected_prefix="☛")]

        if self.option('open'):
            return result.open()


def main():
    """Main entry point"""
    application = Application()
    application.add(CLICommand())
    application.run()



from contextlib import suppress
from .base import BaseSearch


class Skytorrents(BaseSearch):
    """Skytorrents.in torrent search engine."""

    proxy_name = ''
    url = 'https://skytorrents.in/'
    url_format = '{0}{1}search/all/ed/{3}/?l=en-us&q={2}'

    @staticmethod
    def tabulate(row):
        """Extract all information from each href."""
        with suppress(IndexError):
            cols = row.find_all('td')
            links = cols[0].find_all('a')
            return (links[0].text, cols[1].text, links[1]['href'])

    def get_torrents(self):
        """Return torrents."""
        # pylint: disable=not-callable
        return list(
            filter(None,
                   [self.tabulate(l) for l in self.browser.find_all('tr')]))



import os
from .base import BaseSearch

HOST = os.getenv('JACKETT_HOST')
APIKEY = os.getenv('JACKETT_APIKEY')


class Jackett(BaseSearch):
    """Use jackett's all search engines feature"""

    proxy_name = None
    url_format = '{}{}&q={}&p={}'
    url = f"{HOST}/api/v2.0/indexers/all/results/torznab?apikey={APIKEY}"

    def get_torrents(self):
        for item in self.browser.find_all("item"):
            yield (item.find('title').text, item.find('size').text,
                   item.find('enclosure').get('url'))



from .base import BaseSearch


class NyaaSi(BaseSearch):
    """Nyaa.Si torrent search engine."""

    proxy_name = ''
    url = 'https://nyaa.si'
    url_format = '{}{}/?f=0&c=0_0&q={}&p={}'

    @staticmethod
    def tabulate(link):
        """Extract all information from each href."""
        row = link.parent.parent.parent
        cols = row.find_all('td')

        def _nocomments(where):
            return where != 'comments'

        name = ''.join((cols[1].find('a', class_=_nocomments).text.strip(),
                        ' ', cols[0].find('img')['alt'].strip()))
        return (name, cols[3].text.strip(), link.parent['href'])

    def get_torrents(self):
        """Return torrents."""
        # pylint: disable=not-callable
        links = self.browser.find_all(class_='fa-magnet')
        return [self.tabulate(link) for link in links]



from contextlib import suppress
import abc
import itertools

import requests

import robobrowser
import torrentmirror


class BaseSearch(metaclass=abc.ABCMeta):
    """Base Search."""

    def __init__(self, session, logger):
        """Initialize browser instance."""
        self.session = session
        self.logger = logger
        self.browser = robobrowser.RoboBrowser(session=session,
                                               parser='html.parser',
                                               timeout=50)

    @abc.abstractproperty
    def url(self):
        """Search on given url by default."""

    @abc.abstractproperty
    def proxy_name(self):
        """Proxy name to use from torrentmirror."""

    @abc.abstractproperty
    def url_format(self):
        """Url formatter to add to base_url."""

    @abc.abstractmethod
    def get_torrents(self):
        """Return a list of torrents."""

    def search_magnets(self, query: str, page: int):
        """Return the list of magnets from a specific page."""
        proxies = []
        with suppress(Exception):
            proxies = list(
                torrentmirror.get_proxies_for(names=[self.proxy_name]).get(
                    self.proxy_name, []))

        self.logger.debug("Got proxies: %s", proxies)
        proxies.insert(0, [self.url, None])
        for site, _ in proxies:
            self.logger.debug("Searching in %s", site)
            with suppress(requests.exceptions.ReadTimeout,
                          requests.ConnectionError,
                          requests.exceptions.SSLError, AssertionError):
                http = '' if site.startswith('http') else 'http://'
                self.browser.open(
                    self.url_format.format(http, site, query, page))
                torrents = self.get_torrents()
                assert torrents
                self.logger.debug("Got torrents %s", torrents)
                return torrents
        return []

    def search(self, query: str, pagelen: int = 1):
        """Manage complete search for specified pages."""
        return itertools.chain(*(self.search_magnets(query, page + 1)
                                 for page in range(0, pagelen)))



from .base import BaseSearch


class ThePirateBay(BaseSearch):
    """ThePirateBay search engine.

    Search on any of the pirate bay proxies or thepiratebay itself if
    available. Extract torrents, description and return it tabulated and ready
    to be printed
    """

    proxy_name = 'The Pirate Bay'
    url_format = '{}{}/search/{}/{}/99'
    url = 'https://thepiratebay.org/'

    @staticmethod
    def tabulate(link):
        """Make a given a href link parsed on bs4 printable."""
        return (link.parent.find(class_='detName').text,
                link.parent.find(class_='detDesc').text, link['href'])

    def get_torrents(self):
        """Return a list of torrents, printable."""
        # pylint: disable=not-callable
        torrents = self.browser.find_all(
            'a', title='Download this torrent using magnet')
        return [self.tabulate(torrent) for torrent in torrents]



from contextlib import suppress
from .base import BaseSearch


class DigBt(BaseSearch):
    """Digbt DHT search engine."""

    proxy_name = ''
    url = 'https://digbt.org/'
    url_format = '{0}{1}search/{2}-relevance-{3}/'

    @staticmethod
    def tabulate(row):
        """Extract all information from each href."""
        with suppress(IndexError, TypeError):
            tail = row.find(class_='tail')
            return (row.find('div').find('a').text.strip(),
                    list(filter(lambda x: x, tail.text.split(' ')))[4].strip(),
                    tail.find('a')['href'].strip())

    def get_torrents(self):
        """Return torrents."""
        # pylint: disable=not-callable
        return list(
            filter(None,
                   [self.tabulate(l) for l in self.browser.find_all('tr')]))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['katcr', 'katcr.engines']

package_data = \
{'': ['*']}

install_requires = \
['cleo>=0.7.4,<0.8.0',
 'cutie>=0.2.2,<0.3.0',
 'pygogo>=0.12.0,<0.13.0',
 'requests>=2.22,<3.0',
 'robobrowser>=0.5.3,<0.6.0',
 'telepot>=12.7,<13.0',
 'torrentmirror>=2.0,<3.0',
 'werkzeug==0.16.1']

entry_points = \
{'console_scripts': ['katcr = katcr:main']}

setup_kwargs = {
    'name': 'katcr',
    'version': '2.0.1',
    'description': 'KickassTorrents CLI and Telegram bot',
    'long_description': '.. image:: http://i.imgur.com/ofx75lO.png\n\nEasily **search torrents** in multiple providers such as KickAssTorrents and\nThePirateBay.\n\n\n.. image:: https://travis-ci.org/XayOn/katcr.svg?branch=master\n    :target: https://travis-ci.org/XayOn/katcr\n\n.. image:: https://coveralls.io/repos/github/XayOn/katcr/badge.svg?branch=master\n    :target: https://coveralls.io/github/XayOn/katcr?branch=master\n\n.. image:: https://badge.fury.io/py/katcr.svg\n    :target: https://badge.fury.io/py/katcr\n\n\nCommand Line Interface\n----------------------\n\nkatcr comes with a simple but powerful command line interface\n\n.. image:: /docs/screenshot.gif?raw=True\n\n::\n\n   > poetry run katcr search ...\n\n   USAGE\n     katcr search [--pages\xa0<...>] [--token\xa0[<...>]] [--shortener\xa0[<...>]] [--engines\xa0<...>] [--interactive\xa0[<...>]] [--open\xa0[<...>]] <search>\n\n   ARGUMENTS\n     <search>               Search term\n\n   OPTIONS\n     --pages                Pages to search on search engines (default: "1")\n     --token                Token to use on URL shortener as AUTH\n     --shortener            URL Shortener\n     --engines              Engines (default: "Katcr,ThePirateBay,NyaaSi,Skytorrents")\n     --interactive          Allow the user to choose a specific magnet\n     --open                 Open selected magnet with xdg-open\n\n   GLOBAL OPTIONS\n     -h (--help)            Display this help message\n     -q (--quiet)           Do not output any message\n     -v (--verbose)         Increase the verbosity of messages: "-v" for normal output, "-vv" for more verbose output and "-vvv" for debug\n     -V (--version)         Display this application version\n     --ansi                 Force ANSI output\n     --no-ansi              Disable ANSI output\n     -n (--no-interaction)  Do not ask any interactive question\n\n\nInstallation\n------------\n\nThis is a python package available on pypi, just run::\n\n    sudo python3 -m pip install katcr\n\n\nFeatures\n--------\n\n- Display results in a nice utf-8 table\n- Optional interactive mode, choose and open torrent with a nice text user interface\n- Open torrent directly with your preferred client (via xdg-open)\n- Searches on all available engines until it gets results by default.\n- Search torrents in:\n\n  + Katcr\n  + ThePirateBay\n  + Nyaa\n  + Skytorrents\n  + Digbt\n  + `Jackett <https://github.com/Jackett/Jackett>`_\n\n\nJackett Support\n---------------\n\nYou can easily use a `Jackett <https://github.com/Jackett/Jackett>`_ instance\nto search on all your configured provider.\n\nThis allows you to search on any jackett-supported site (that\'s about supported\n300 trackers).\n\nJackett is probably the best way to use katcr and katbot, as it has a more\nactive mantainance of the tracker sites that us.\n\nTo enable Jackett use, simply export your jackett URL and TOKEN as variables::\n\n   JACKETT_HOST=http://127.0.0.1:9117 JACKETT_APIKEY=<redacted> poetry run katcr --engines=\n\n\n\nKATBot - Kickasstorrents telegram bot\n--------------------------------------\n\nKatcr also comes with a telegram bot entry point.\nIt\'s a simple bot that replies with search results for each message it gets.\n\nIt returns magnet links with provided url shortener for the first page of\nresults, on the first available search engine.\n\n.. image:: http://i.imgur.com/7FxplBs.gif\n\n::\n\n    Run telegram bot.\n\n        Usage: katcr_bot [options]\n\n        Options:\n            --token=<BOT_TOKEN>             Telegram bot token\n            --token-file=<FILE>             Telegram bot token file\n            --shortener=<URL_SHORTENER>     Url shortener to use\n                                            [default: http://shortmag.net]\n\n\nNotes\n------\n\nThis project is made with the best of intentions.\n\nFor that times you need to search for somethink shared as a torrent on KAT\n(I.E, linux images). Logo is based on robot cat by\n`Arsenty <https://thenounproject.com/arsenty/>`_\n\nIf you like this project, show its appreciation by starring it, if you\'re using\nit and want to write to me personally, feel free to do so at\nopensource@davidfrancos.net. If you\'ve got a bug to report, please use the\ngithub ticketing system\nPending things\n--------------\n\n* Fix tests\n* Add information about seeds/leeches on each torrent\n* Add more search engines\n* Maybe direct search to the DHT?\n\n',
    'author': 'David Francos',
    'author_email': 'opensource@davidfrancos.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
